#ifndef __cothread_H
#define __cothread_H

#include "kernel/types.h"
#include "kernel/stat.h"
#include "user/user.h"

// cothread states
enum cothread_state { FREE, READY, FINISHED, RUNNING };

#define STACK_SIZE  4096
#define MAX_cothread  4

// cothread structure
struct cothread {
    uint64 s[12];             // Used for saving s registers
    uint64 ra;                // Return address
    uint64 sp;                // Stack pointer
    char stack[STACK_SIZE];   // The cothread's stack
    int state;                // FREE, RUNNING, READY
};

// Global variables
struct cothread all_cothread[MAX_cothread];
struct cothread *current_cothread;

// Function declarations
void cothread_switch(uint64 old_c_ptr, uint64 new_c_ptr);
void cothread_init(void);
void cothread_schedule(void);
void cothread_create(void (*func)());
void cothread_yield(void);

#endif // __cothread_H