#ifndef __USERTHREAD_H
#define __USERTHREAD_H

#include "kernel/types.h"
#include "kernel/stat.h"
#include "user/user.h"
#include "user/userlock.h"

enum thread_state { READY, RUNNING, FINISHED };

#define PGSIZE      4096
#define MAXTHREADNUM   20

struct threadContext{
    uint64 ra;
    uint64 sp;
    uint64 s0;
    uint64 s1;
    uint64 s2;
    uint64 s3;
    uint64 s4;
    uint64 s5;
    uint64 s6;
    uint64 s7;
    uint64 s8;
    uint64 s9;
    uint64 s10;
    uint64 s11;
    uint64 t0;
    uint64 t1;
    uint64 t2;
    uint64 t3;
    uint64 t4;
    uint64 t5;
    uint64 t6;
    uint64 a0;
    uint64 a1;
    uint64 a2;
    uint64 a3;
    uint64 a4;
    uint64 a5;
    uint64 a6;
    uint64 a7;
    uint64 epc;
};


/*
    We allocate one page for each thread, where the thread struct is put at 
         the low address and the stack pointer is set to the top of the page.
    The memory layout is illustrated in the graph:

                    ┌─────────────┐◄─── High Address
                    │             │
                    │    stack    │
                    │             │
                    │             │
              sp ──►├──────┬──────┤
                    │      │      │
                    │      ▼      │
                    │             │
                    │             │
                    │             │
                    │             │
    4KiB Page       │             │
                    │             │
                    │             │
                    │             │
                    ├─────────────┤
                    │             │
                    │             │
                    │thread_struct│
                    │             │
                    │             │
                    └─────────────┘◄─── Low Address
*/

//use doubly linked list to manage free slots in threadpool's queue

struct thread{
    int id;
    struct threadContext context;           /*THREAD CONTEXT SAVED HERE*/
    int state;                              /*FREE, RUNNING OR RUNNABLE*/
};

// We allocate thread struct lazily.
struct threadpool{
    uint32 thread_count;                            /*HOLDS THE NUMBER OF ACTIVE THREADS*/
    struct thread *queue[MAXTHREADNUM];             /*HOLDS POINTERS TO ALL THREAD STRUCT*/
    int currentThreadId;
};

extern struct threadpool pool;

void print_context(struct threadContext *p); // debug only
int uthread_create(void (*func)(void *), void *arg);
void uthread_init(void);                             /*initialize the thread pool*/
void uthread_run(void);
uint32 uthread_id(void);
void uthread_scheduler(void);
void handler(void* u_context_ptr);
#endif

// //clear all the finished threads.
// void uthread_clear(){
//     int i;
//     for(i = 0; i < pool.thread_count ; i++){
//         if(pool.queue[i]->state == FINISHED){
//             free((void*) pool.queue[i]);
//             if(i != pool.thread_count - 1)
//                 pool.queue[FINISHED] = pool.queue[pool.thread_count - 1];
            
//         }
//     }
// }
